@@ -21,21 +21,18 @@
  * 02111-1307, USA.
  *)
 
-open Lwt
+let (>>=) = Lwt.(>>=)
 
 type t = { mutable locked : bool; mutable waiters : unit Lwt.u Lwt_sequence.t  }
 
 let create () = { locked = false; waiters = Lwt_sequence.create () }
 
 let rec lock m =
-  if m.locked then begin
-    let (res, w) = Lwt.task () in
-    let node = Lwt_sequence.add_r w m.waiters in
-    Lwt.on_cancel res (fun _ -> Lwt_sequence.remove node);
-    res
-  end else begin
+  if m.locked then
+    Lwt.add_task_r m.waiters
+  else begin
     m.locked <- true;
-    Lwt.return ()
+    Lwt.return_unit
   end
 
 let unlock m =
@@ -49,12 +46,8 @@ let unlock m =
   end
 
 let with_lock m f =
-  lwt () = lock m in
-  try_lwt
-    f ()
-  finally
-    unlock m;
-    return ()
+  lock m >>= fun () ->
+  Lwt.finalize f (fun () -> unlock m; Lwt.return_unit)
 
 let is_locked m = m.locked
 let is_empty m = Lwt_sequence.is_empty m.waiters
