@@ -129,10 +129,11 @@ CAMLprim value lwt_glib_get_sources()
 {
   gint timeout;
   int i;
+  int events;
   GPollFD *gpollfd;
 
   CAMLparam0();
-  CAMLlocal4(fd, fds, src, result);
+  CAMLlocal3(fds, watches, result);
 
   g_main_context_dispatch(gc);
   g_main_context_prepare(gc, &max_priority);
@@ -144,28 +145,30 @@ CAMLprim value lwt_glib_get_sources()
   }
 
   fds = caml_alloc_tuple(n_fds);
+  watches = caml_alloc_tuple(n_fds);
   for (i = 0; i < n_fds; i++) {
     gpollfd = gpollfds + i;
     gpollfd->revents = 0;
 
+    events = 0;
+    if (gpollfd->events & G_IO_IN) events |= 1;
+    if (gpollfd->events & G_IO_OUT) events |= 2;
+
 #if defined(LWT_ON_WINDOWS)
     /* On windows, glib file descriptors are handles */
-    fd = alloc_fd((HANDLE)gpollfd->fd);
+    Field(fds, i) = alloc_fd((HANDLE)gpollfd->fd);
 #else
-    fd = Val_int(gpollfd->fd);
+    Field(fds, i) = Val_int(gpollfd->fd);
+    if (gpollfd->fd < 0) events = 0;
 #endif
 
-    src = caml_alloc_tuple(3);
-    Field(src, 0) = fd;
-    Field(src, 1) = Val_bool(gpollfd->events & G_IO_IN);
-    Field(src, 2) = Val_bool(gpollfd->events & G_IO_OUT);
-
-    Field(fds, i) = src;
+    Field(watches, i) = Val_int(events);
   }
 
-  result = caml_alloc_tuple(2);
+  result = caml_alloc_tuple(3);
   Store_field(result, 0, fds);
-  Store_field(result, 1, caml_copy_double(timeout * 1e-3));
+  Store_field(result, 1, watches);
+  Store_field(result, 2, caml_copy_double(timeout * 1e-3));
 
   CAMLreturn(result);
 }
