@@ -21,12 +21,19 @@
  * 02111-1307, USA.
  *)
 
+let (>>=) = Lwt.bind
+
 type t =
     Plain
   | SSL of Ssl.socket
 
 type socket = Lwt_unix.file_descr * t
 
+let ssl_socket (fd, kind) =
+  match kind with
+    | Plain -> None
+    | SSL socket -> Some socket
+
 let is_ssl s =
   match snd s with
     Plain -> false
@@ -145,12 +152,6 @@ let wait_write (fd, s) =
     Plain -> Lwt_unix.wait_write fd
   | SSL _ -> Lwt_unix.yield ()
 
-let out_channel_of_descr s =
-  Lwt_io.make ~mode:Lwt_io.output (fun buf pos len -> write_bytes s buf pos len)
-
-let in_channel_of_descr s =
-  Lwt_io.make ~mode:Lwt_io.input (fun buf pos len -> read_bytes s buf pos len)
-
 let ssl_shutdown (fd, s) =
   match s with
     Plain -> Lwt.return ()
@@ -162,6 +163,23 @@ let close (fd, _) = Lwt_unix.close fd
 
 let abort (fd, _) = Lwt_unix.abort fd
 
+let shutdown_and_close s =
+  ssl_shutdown s >>= fun () ->
+  Lwt.wrap2 shutdown s Unix.SHUTDOWN_ALL >>= fun () ->
+  close s
+
+let out_channel_of_descr s =
+  Lwt_io.make
+    ~mode:Lwt_io.output
+    ~close:(fun () -> shutdown_and_close s)
+    (fun buf pos len -> write_bytes s buf pos len)
+
+let in_channel_of_descr s =
+  Lwt_io.make
+    ~mode:Lwt_io.input
+    ~close:(fun () -> shutdown_and_close s)
+    (fun buf pos len -> read_bytes s buf pos len)
+
 let get_fd (fd,socket) =
   match socket with
     | Plain -> Lwt_unix.unix_file_descr fd
