@@ -137,6 +137,7 @@ struct
     | Complete_first
     | Complete_last
     | Undo
+    | Delete_char_or_list
 
   let names = [
     (Nop, "nop");
@@ -176,6 +177,7 @@ struct
     (Complete_first, "complete-first");
     (Complete_last, "complete-last");
     (Undo, "undo");
+    (Delete_char_or_list, "delete-char-or-list");
   ]
 
   let to_string = function
@@ -214,7 +216,7 @@ struct
     | Key_delete -> Forward_delete_char
     | Key_control '@' -> Set_mark
     | Key_control 'a' -> Beginning_of_line
-    | Key_control 'd' -> Break
+    | Key_control 'd' -> Delete_char_or_list
     | Key_control 'e' -> End_of_line
     | Key_control 'h' -> Backward_delete_word
     | Key_control 'i' -> Complete
@@ -1076,7 +1078,7 @@ struct
 
     (* Thread of the last [read_command]. It is cancelled when
        read-line terminates. *)
-    let last_read_command_thread = ref (fail Exit) in
+    let last_read_command_thread = ref (raise_lwt Exit) in
 
     (* Events typed by the user: *)
     let user_events = Lwt_stream.from (fun () ->
@@ -1199,14 +1201,14 @@ struct
                   process_event state event (loop prev)
 
               | Fail exn ->
-                  fail exn
+                  raise_lwt exn
             end
 
         | Return event ->
             process_event state event (loop prev)
 
         | Fail exn ->
-            fail exn
+            raise_lwt exn
 
     (* loop_refresh redraw the current state, even if it haa not
        changed: *)
@@ -1225,14 +1227,14 @@ struct
                   process_event state event loop_refresh
 
               | Fail exn ->
-                  fail exn
+                  raise_lwt exn
             end
 
         | Return event ->
             process_event state event loop_refresh
 
         | Fail exn ->
-            fail exn
+            raise_lwt exn
 
     and process_event state event loop = match event with
       | Ev_prompt prompt ->
@@ -1302,7 +1304,18 @@ struct
                   state
           in
 
-          filter state command >>= function
+          (* User-provided filter *)
+          lwt command = filter state command in
+
+          (* Commands that need pre-processing *)
+          lwt command = match command, state.engine.Engine.mode with
+            | Delete_char_or_list, Engine.Edition ("", "") -> return Break
+            | Delete_char_or_list, Engine.Edition (_, "") -> return Complete
+            | Delete_char_or_list, Engine.Edition (_, _) -> return Forward_delete_char
+            | _ -> return command
+          in
+
+          match command with
             | Nop ->
                 loop state
 
@@ -1384,7 +1397,7 @@ struct
                             loop { state with engine = { state.engine with Engine.mode = Engine.Edition(before ^ Text.sub word idx (word_len - idx), after) } }
                           else
                             match Text.next ptr with
-                              | None -> fail (Failure "invalid completion")
+                              | None -> raise_lwt (Failure "invalid completion")
                               | Some(ch, ptr) -> search ptr (idx - 1)
                         in
                         if word_len > before_len then
@@ -1444,7 +1457,7 @@ struct
         | `Accept ->
             map_result (Engine.all_input state.engine)
         | `Interrupt ->
-            fail Interrupt
+            raise_lwt Interrupt
     end in
     {
       result = result;
@@ -1466,14 +1479,18 @@ struct
   let make_prompt prompt = React.S.value (prompt (React.S.const (Engine.init [])))
 
   let read_line ?history ?complete ?clipboard ?mode ?(prompt=default_prompt) () =
-    if Unix.isatty Unix.stdin && Unix.isatty Unix.stdout then
-      make ?history ?complete ?clipboard ?mode ~prompt ~map_result:return ()
+    lwt stdin_isatty = Lwt_unix.isatty Lwt_unix.stdin
+    and stdout_isatty = Lwt_unix.isatty Lwt_unix.stdout in
+    if stdin_isatty && stdout_isatty then
+      return (make ?history ?complete ?clipboard ?mode ~prompt ~map_result:return ())
     else
-      fake (lwt () = write stdout (strip_styles (make_prompt prompt)) in
-            Lwt_text.read_line stdin)
+      return (fake (lwt () = write stdout (strip_styles (make_prompt prompt)) in
+                    Lwt_text.read_line stdin))
 
   let read_password ?clipboard ?(style:password_style=`text "*") ?prompt () =
-    if Unix.isatty Unix.stdin && Unix.isatty Unix.stdout then
+    lwt stdin_isatty = Lwt_unix.isatty Lwt_unix.stdin
+    and stdout_isatty = Lwt_unix.isatty Lwt_unix.stdout in
+    if stdin_isatty && stdout_isatty then
       let map_text = match style with
         | `text ch -> (fun txt -> Text.map (fun _ -> ch) txt)
         | `clear -> (fun x -> x)
@@ -1485,9 +1502,9 @@ struct
         | command ->
             return command
       in
-      make ?clipboard ~map_text ~mode:`none ~filter ?prompt ~map_result:return ()
+      return (make ?clipboard ~map_text ~mode:`none ~filter ?prompt ~map_result:return ())
     else
-      failwith "Lwt_read_line.read_password: not running in a terminal"
+      fail (Failure "Lwt_read_line.read_password: not running in a terminal")
 
   let read_keyword ?history ?(case_sensitive=false) ?mode ?(prompt=default_prompt) ~values () =
     let compare = if case_sensitive then Text.compare else Text.icompare in
@@ -1499,7 +1516,9 @@ struct
           else
             assoc key l
     in
-    if Unix.isatty Unix.stdin && Unix.isatty Unix.stdout then
+    lwt stdin_isatty = Lwt_unix.isatty Lwt_unix.stdin
+    and stdout_isatty = Lwt_unix.isatty Lwt_unix.stdout in
+    if stdin_isatty && stdout_isatty then
       let words = List.fold_left (fun acc (key, value) -> TextSet.add key acc) TextSet.empty values in
       let filter state = function
         | Accept_line ->
@@ -1518,15 +1537,15 @@ struct
       and complete (before, after) =
         return (complete "" before after words)
       in
-      make ?history ?mode ~prompt ~filter ~map_result ~complete ()
+      return (make ?history ?mode ~prompt ~filter ~map_result ~complete ())
     else
-      fake (lwt () = write stdout (strip_styles (make_prompt prompt)) in
-            lwt txt = Lwt_text.read_line stdin in
-            match assoc txt values with
-              | Some value ->
-                  return value
-              | None ->
-                  fail (Failure "Lwt_read_line.read_keyword: invalid input"))
+      return (fake (lwt () = write stdout (strip_styles (make_prompt prompt)) in
+                    lwt txt = Lwt_text.read_line stdin in
+                    match assoc txt values with
+                      | Some value ->
+                          return value
+                      | None ->
+                          raise_lwt (Failure "Lwt_read_line.read_keyword: invalid input")))
 
   let read_yes_no ?history ?mode ?prompt () =
     read_keyword ?history ?mode ?prompt ~values:[("yes", true); ("no", false)] ()
@@ -1539,20 +1558,16 @@ end
 let default_prompt = [Text "# "]
 
 let read_line ?history ?complete ?clipboard ?mode ?(prompt=default_prompt) () =
-  Control.result
-    (Control.read_line ?history ?complete ?clipboard ?mode ~prompt:(fun _ -> React.S.const prompt) ())
+  Control.read_line ?history ?complete ?clipboard ?mode ~prompt:(fun _ -> React.S.const prompt) () >>= Control.result
 
 let read_password ?clipboard ?style ?(prompt=default_prompt) () =
-  Control.result
-    (Control.read_password ?clipboard ?style ~prompt:(fun _ -> React.S.const prompt) ())
+  Control.read_password ?clipboard ?style ~prompt:(fun _ -> React.S.const prompt) () >>= Control.result
 
 let read_keyword ?history ?case_sensitive ?mode ?(prompt=default_prompt) ~values () =
-  Control.result
-    (Control.read_keyword ?history ?case_sensitive ?mode ~prompt:(fun _ -> React.S.const prompt) ~values ())
+  Control.read_keyword ?history ?case_sensitive ?mode ~prompt:(fun _ -> React.S.const prompt) ~values () >>= Control.result
 
 let read_yes_no ?history ?mode ?(prompt=default_prompt) () =
-  Control.result
-    (Control.read_yes_no ?history ?mode ~prompt:(fun _ -> React.S.const prompt) ())
+  Control.read_yes_no ?history ?mode ~prompt:(fun _ -> React.S.const prompt) () >>= Control.result
 
 (* +-----------------------------------------------------------------+
    | History                                                         |
